"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fm = require("front-matter");
const marked = require("marked");
const globby = require("globby");
const util = require("./util");
const get_image_meta_1 = require("./get-image-meta");
const getDirName = (pathName) => path
    .dirname(pathName)
    .split(path.sep)
    .pop();
const getDataForImage = (albumsDir, albumName) => (file) => ({
    filePath: file,
    detailLinkProps: util.getDetailLinkProps(albumsDir, albumName, file),
    fileName: path.basename(file),
});
const getMetaFromImage = (image) => __awaiter(this, void 0, void 0, function* () {
    const meta = yield get_image_meta_1.default(image.filePath);
    return Object.assign({}, image, { meta });
});
const getImages = (albumsDir, albumName) => __awaiter(this, void 0, void 0, function* () {
    const pattern = path.join('static', albumsDir, albumName, '*.jpg');
    const files = yield globby(pattern);
    const images = files.map(getDataForImage(albumsDir, albumName));
    return yield Promise.all(images.map(getMetaFromImage));
});
const getPageContent = (contentDir, pagePath, pathPrefix = '') => __awaiter(this, void 0, void 0, function* () {
    const content = yield util.pReadFile(pagePath);
    const { attributes, body } = fm(String(content));
    const name = getDirName(pagePath);
    return {
        meta: attributes,
        path: `/${pathPrefix}${name}/`,
        body: marked(body),
        name: name === 'content' ? 'frontpage' : name,
    };
});
const gatherAlbumData = (albumsDir, contentDir) => (file) => __awaiter(this, void 0, void 0, function* () {
    const albumName = file
        .split(path.sep)
        .splice(-2)
        .shift();
    const [content, images] = yield Promise.all([
        getPageContent(contentDir, file, `${albumsDir}/`),
        getImages(albumsDir, albumName),
    ]);
    return {
        content,
        images,
    };
});
const getPages = (pagesGlob, albumsGlob, contentDir) => __awaiter(this, void 0, void 0, function* () {
    const files = yield globby(pagesGlob, { ignore: albumsGlob });
    return yield Promise.all(files.map((file) => __awaiter(this, void 0, void 0, function* () { return getPageContent(contentDir, file); })));
});
exports.default = (config) => __awaiter(this, void 0, void 0, function* () {
    const patterns = util.getGlobPatterns(config);
    const albumFiles = yield globby(patterns.albums);
    const albums = yield Promise.all(albumFiles.map(gatherAlbumData(config.albumsDir, config.contentDir)));
    const pages = yield getPages(patterns.pages, patterns.albums, config.contentDir);
    return { albums, pages };
});
