"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const globby = require("globby");
const config_1 = require("./config");
const util_1 = require("./util");
const removeSlashes = (str) => str.replace(/^\/|\/$/g, '');
const getPaths = (prefix, pattern, ignore) => __awaiter(this, void 0, void 0, function* () {
    const paths = yield globby(pattern, ignore ? { ignore } : null);
    return (paths
        .map(path_1.dirname)
        .map((x) => path_1.basename(x))
        .map((x) => `/${x}/`));
});
const getImagePaths = (pattern, albumsDir) => __awaiter(this, void 0, void 0, function* () {
    const images = yield globby(pattern);
    return images.map((filePath) => {
        const regExp = new RegExp(`^.*static/${albumsDir}`);
        return filePath.replace(regExp, '').replace(/\.(jpe?g|png|gif)$/, '/');
    });
});
const getAlbums = (albumsDir, albums) => albums.reduce((acc, album) => (Object.assign({}, acc, { [`/${albumsDir}${album}`]: {
        query: {
            album: removeSlashes(album),
        },
        page: '/album',
    } })), {});
const getPages = (pages) => pages.reduce((acc, page) => (Object.assign({}, acc, { [page]: {
        query: {
            page: removeSlashes(page),
        },
        page: '/default',
    } })), {});
const getImageQueryFromPath = (image) => {
    const parts = image.split('/').filter(Boolean);
    return {
        album: parts.shift(),
        image: parts.shift(),
    };
};
const getImages = (albumsDir, images) => images.reduce((acc, image) => (Object.assign({}, acc, { [`/${albumsDir}${image}`]: {
        query: getImageQueryFromPath(image),
        page: '/image',
    } })), {});
exports.default = () => __awaiter(this, void 0, void 0, function* () {
    const conf = config_1.default();
    const imagesPattern = path_1.join('static', conf.albumsDir, '**/*.{jpg,png,gif}');
    const patterns = util_1.getGlobPatterns(conf);
    const albumPaths = yield getPaths(path_1.join(conf.contentDir, conf.albumsDir), patterns.albums);
    const imagePaths = yield getImagePaths(imagesPattern, conf.albumsDir);
    const pagePaths = yield getPaths(conf.contentDir, patterns.pages, patterns.albums);
    const pagePathsExceptIndex = pagePaths.filter((x) => x !== `/${conf.contentDir}/`);
    const [index] = yield globby(path_1.join(conf.contentDir, 'index.md'));
    if (!index) {
        throw Error('Please define a frontpage.');
    }
    return Object.assign({
        '/': { page: '/' },
    }, getAlbums(conf.albumsDir, albumPaths), getPages(pagePathsExceptIndex), getImages(conf.albumsDir, imagePaths));
});
