"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const next = require("next");
const config_1 = require("./config");
const get_data_1 = require("./get-data");
const common_1 = require("./handlers/common");
const frontpage_data_1 = require("./handlers/frontpage-data");
const album_data_1 = require("./handlers/album-data");
const image_data_1 = require("./handlers/image-data");
const config = config_1.default();
const quiet = true;
const dev = process.env.NODE_ENV !== 'production';
const app = next({ dev, quiet });
const handle = app.getRequestHandler();
exports.default = () => __awaiter(this, void 0, void 0, function* () {
    yield app.prepare();
    const { albums, pages } = yield get_data_1.default(config);
    const server = express();
    server.get('/', common_1.default(app, '/index'));
    server.get(`/${config.albumsDir}/:album/`, common_1.default(app, '/album'));
    server.get(`/${config.albumsDir}/:album/:image/`, common_1.default(app, '/image'));
    server.get('/:page/', common_1.default(app, '/default'));
    server.get('/data/index.json', frontpage_data_1.default(pages, albums, config.albumsDir));
    server.get(`/data/${config.albumsDir}/(:album).json`, album_data_1.default(albums));
    server.get(`/data/${config.albumsDir}/(:album)/(:image).json`, image_data_1.default(config.albumsDir, albums, app));
    server.get('/data/(:page).json', (req, res) => res.json(pages.find((page) => page.name === req.params.page)));
    // tslint:disable-next-line:no-unnecessary-callback-wrapper
    server.get('*', (req, res) => {
        // tslint:disable-next-line:no-floating-promises
        handle(req, res);
    });
    return { server, app };
});
